@@ -38,11 +38,13 @@ from osc_lib import exceptions as exc
 from osc_lib.i18n import _
 from osc_lib import logs
 from osc_lib import utils
+from osc_lib import version
 
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
 
 DEFAULT_DOMAIN = 'default'
+DEFAULT_INTERFACE = 'public'
 
 
 def prompt_for_password(prompt=None):
@@ -153,14 +155,19 @@ class OpenStackShell(app.App):
 
     def close_profile(self):
         if self.do_profile:
-            trace_id = osprofiler_profiler.get().get_base_id()
+            profiler = osprofiler_profiler.get()
+            trace_id = profiler.get_base_id()
+            # Short ID for OpenTracing-based driver (64-bit id)
+            short_id = profiler.get_shorten_id(trace_id)
 
             # NOTE(dbelova): let's use warning log level to see these messages
             # printed. In fact we can define custom log level here with value
             # bigger than most big default one (CRITICAL) or something like
             # that (PROFILE = 60 for instance), but not sure we need it here.
             self.log.warning("Trace ID: %s" % trace_id)
-            self.log.warning("Display trace with command:\n"
+            self.log.warning("Short trace ID "
+                             "for OpenTracing-based drivers: %s" % short_id)
+            self.log.warning("Display trace data with command:\n"
                              "osprofiler trace show --html %s " % trace_id)
 
     def run_subcommand(self, argv):
@@ -249,10 +256,57 @@ class OpenStackShell(app.App):
             metavar='<interface>',
             dest='interface',
             choices=['admin', 'public', 'internal'],
-            default=utils.env('OS_INTERFACE'),
+            default=utils.env(
+                'OS_INTERFACE',
+                default=DEFAULT_INTERFACE),
             help=_('Select an interface type.'
                    ' Valid interface types: [admin, public, internal].'
-                   ' (Env: OS_INTERFACE)'),
+                   ' default=%s, (Env: OS_INTERFACE)') % DEFAULT_INTERFACE,
+        )
+        parser.add_argument(
+            '--os-service-provider',
+            metavar='<service_provider>',
+            dest='service_provider',
+            default=utils.env('OS_SERVICE_PROVIDER'),
+            help=_('Authenticate with and perform the command on a service'
+                   ' provider using Keystone-to-keystone federation. Must'
+                   ' also specify the remote project option.')
+        )
+        remote_project_group = parser.add_mutually_exclusive_group()
+        remote_project_group.add_argument(
+            '--os-remote-project-name',
+            metavar='<remote_project_name>',
+            dest='remote_project_name',
+            default=utils.env('OS_REMOTE_PROJECT_NAME'),
+            help=_('Project name when authenticating to a service provider'
+                   ' if using Keystone-to-Keystone federation.')
+        )
+        remote_project_group.add_argument(
+            '--os-remote-project-id',
+            metavar='<remote_project_id>',
+            dest='remote_project_id',
+            default=utils.env('OS_REMOTE_PROJECT_ID'),
+            help=_('Project ID when authenticating to a service provider'
+                   ' if using Keystone-to-Keystone federation.')
+        )
+        remote_project_domain_group = parser.add_mutually_exclusive_group()
+        remote_project_domain_group.add_argument(
+            '--os-remote-project-domain-name',
+            metavar='<remote_project_domain_name>',
+            dest='remote_project_domain_name',
+            default=utils.env('OS_REMOTE_PROJECT_DOMAIN_NAME'),
+            help=_('Domain name of the project when authenticating to a'
+                   ' service provider if using Keystone-to-Keystone'
+                   ' federation.')
+        )
+        remote_project_domain_group.add_argument(
+            '--os-remote-project-domain-id',
+            metavar='<remote_project_domain_id>',
+            dest='remote_project_domain_id',
+            default=utils.env('OS_REMOTE_PROJECT_DOMAIN_ID'),
+            help=_('Domain ID of the project when authenticating to a'
+                   ' service provider if using Keystone-to-Keystone'
+                   ' federation.')
         )
         parser.add_argument(
             '--timing',
@@ -286,7 +340,7 @@ class OpenStackShell(app.App):
     * super()
     * _final_defaults()
     * OpenStackConfig
-    * get_one_cloud
+    * get_one
     * _load_plugins()
     * _load_commands()
     * ClientManager
@@ -344,7 +398,7 @@ class OpenStackShell(app.App):
         # Parent __init__ parses argv into self.options
         super(OpenStackShell, self).initialize_app(argv)
         self.log.info("START with options: %s",
-                      strutils.mask_password(self.command_options))
+                      strutils.mask_password(" ".join(self.command_options)))
         self.log.debug("options: %s",
                        strutils.mask_password(self.options))
 
@@ -352,12 +406,10 @@ class OpenStackShell(app.App):
         self._final_defaults()
 
         # Do configuration file handling
-        # Ignore the default value of interface. Only if it is set later
-        # will it be used.
         try:
             self.cloud_config = cloud_config.OSC_Config(
+                pw_func=prompt_for_password,
                 override_defaults={
-                    'interface': None,
                     'auth_type': self._auth_type,
                 },
             )
@@ -373,7 +425,7 @@ class OpenStackShell(app.App):
 
         # NOTE(dtroyer): Need to do this with validate=False to defer the
         #                auth plugin handling to ClientManager.setup_auth()
-        self.cloud = self.cloud_config.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=False,
@@ -412,7 +464,7 @@ class OpenStackShell(app.App):
         )
 
         # NOTE(dtroyer): If auth is not required for a command, skip
-        #                get_one_Cloud()'s validation to avoid loading plugins
+        #                get_one()'s validation to avoid loading plugins
         validate = cmd.auth_required
 
         # NOTE(dtroyer): Save the auth required state of the _current_ command
@@ -420,10 +472,13 @@ class OpenStackShell(app.App):
         self.client_manager._auth_required = cmd.auth_required
 
         # Validate auth options
-        self.cloud = self.cloud_config.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=validate,
+            app_name=self.client_manager._app_name,
+            app_version=self.client_manager._app_version,
+            additional_user_agent=[('osc-lib', version.version_string)],
         )
         # Push the updated args into ClientManager
         self.client_manager._cli_options = self.cloud
